@@ -31,9 +31,11 @@
      Rationalize common code for 32/64 bit architectures.
      Fix error message for modules.dep.
        Keith Owens <kaos@ocs.com.au> December 1999
+     Add arch64().
+       Keith Owens <kaos@ocs.com.au> December 1999.
    */
 
-#ident "$Id: depmod.c 1.7 Mon, 20 Dec 1999 12:34:51 +1100 keith $"
+#ident "$Id: depmod.c 1.9 Wed, 15 Mar 2000 21:14:05 +1100 keith $"
 
 #include <stdio.h>
 #include <stdlib.h>
@@ -49,7 +51,6 @@
 #include <syslog.h>
 #include <elf.h>
 #include ELF_MACHINE_H
-#include <sys/utsname.h>
 
 #include "module.h"
 #include "version.h"
@@ -106,6 +107,8 @@ extern int depmod_main(int argc, char **argv);
 extern int depmod_main_32(int argc, char **argv);
 extern int depmod_main_64(int argc, char **argv);
 
+extern int quick;	/* Option -A */
+
 
 /*
  *	Create a symbol definition.
@@ -555,7 +558,7 @@ void depmod_usage(void)
 {
 	fprintf(stderr,
 	"depmod " MODUTILS_VERSION "\n"
-	"depmod -a [-n -e -v -q -V]\n"
+	"depmod -[aA] [-n -e -v -q -V]\n"
 	"      [-C configfile] [-F kernelsyms] [-b basedirectory] [forced_version]\n"
 	"depmod [-n -e -v -q] [-f kernelsyms] module1.o module2.o ...\n"
 	"If no arguments (expect options) are given, \"depmod -a\" is assumed\n"
@@ -565,6 +568,9 @@ void depmod_usage(void)
 	ETC_MODULES_CONF ". It will output the result into the depfile specified\n"
 	"in this configuration file\n"
 	"\n"
+	"depmod -A is the same as depmod -a, but will first compare the timestamps\n"
+	"of the files involved to see if the depfile needs updating.\n"
+	"\n"
 	"Normally depmod operates silently, reporting only the list of modules that\n"
 	"won't load properly (missing symbols).\n"
 	"Option -q will make depmod keep quiet about missing symbols\n"
@@ -595,8 +601,7 @@ void depmod_usage(void)
 #if defined(COMMON_3264) && defined(ONLY_64)
 int depmod_main(int argc, char **argv)
 {
-        struct utsname u;
-        if (!uname (&u) && !strcmp (u.machine, "64"))
+        if (arch64())
                 return depmod_main_64(argc, argv);
         else
                 return depmod_main_32(argc, argv);
@@ -632,10 +637,12 @@ int DEPMOD_MAIN(int argc, char **argv)
 
 	error_file = "depmod";
 
-	while ((o = getopt_long(argc, argv, "ab:C:eF:hnqsvVr",
+	while ((o = getopt_long(argc, argv, "aAb:C:eF:hnqsvVr",
 				&long_opts[0], NULL)) != EOF) {
 		switch (o) {
+		case 'A':
 		case 'a':
+			quick = o == 'A';
 			stdmode = 1;	/* Probe standard directory */
 			break;		/* using the config file */
 
