@@ -2,7 +2,7 @@
 /* Command-line: gperf -a -C -g -H is_reserved_hash -k 1,3,$ -N is_reserved_word -p -t keywords.gperf  */
 
 
-#ident "$Id: keywords.c 1.1 Wed, 25 Aug 1999 16:26:49 +1000 keith $"
+#ident "$Id: keywords.c 1.2 Sun, 21 Nov 1999 19:27:16 +1100 keith $"
 struct resword { const char *name; int token; };
 
 #define MIN_WORD_LENGTH 3
@@ -36,7 +36,7 @@ is_reserved_hash (register const char *str, register int len)
      15, 20,  5, 65, 10,  0,  0, 25,  5, 65,
      65, 65, 65, 65, 65, 65, 65, 65,
   };
-  return len + hash_table[str[2]] + hash_table[str[0]] + hash_table[str[len - 1]];
+  return len + hash_table[(int)str[2]] + hash_table[(int)str[0]] + hash_table[(int)str[len - 1]];
 }
 
 #ifdef __GNUC__
