@@ -42,9 +42,11 @@
  
      Rationalize common code for 32/64 bit architectures.
        Keith Owens <kaos@ocs.com.au> December 1999.
+     Add arch64().
+       Keith Owens <kaos@ocs.com.au> December 1999.
    */
 
-#ident "$Id: insmod.c 1.10 Sun, 19 Dec 1999 20:23:21 +1100 keith $"
+#ident "$Id: insmod.c 1.12 Wed, 15 Mar 2000 23:50:08 +1100 keith $"
 
 #include <sys/types.h>
 #include <stdlib.h>
@@ -58,7 +60,6 @@
 #include <getopt.h>
 #include <sys/stat.h>
 #include <sys/file.h>
-#include <sys/utsname.h>
 
 #include "module.h"
 #include "obj.h"
@@ -118,28 +119,58 @@ static int get_kernel_version(char str[STRVERSIONLEN])
 /* String comparison for non-co-versioned kernel and module. 
  * prefix should be the same as used by genksyms for this kernel.
  */
-#define SMP_PREFIX "smp_"
-static char *ncv_prefix = "";	/* Overridden by --prefix option */
+static char *ncv_prefix = NULL;	/* Overridden by --prefix option */
 static int ncv_plen = 0;
 
-/* Only set prefix once. If set from command line, do nothing. */
+/* Only set prefix once. If set by the user, use it.  If not set by the
+ * user, look for a well known kernel symbol and derive the prefix from
+ * there.  Otherwise set the prefix depending on whether uts_info
+ * includes SMP or not for backwards compatibility.
+ */
 static void set_ncv_prefix(char *prefix)
 {
-	static int prefix_set = 0;
+	static char derived_prefix[256];
+	static const char well_known_symbol[] = "get_module_symbol_R";
+	struct module_symbol *s;
+	int i, l, pl;
+	const char *name;
+	char *p;
 
-	if (prefix_set)
+	if (ncv_prefix)
 		return;
 
-	prefix_set = 1;
-
 	if (prefix)
 		ncv_prefix = prefix;
 	else {
-		char *p = strchr(uts_info.version, ' ');
+		/* Extract the prefix (if any) from the well known symbol */
+		for (i = 0, s = ksyms; i < nksyms; ++i, ++s) {
+			name = (char *) s->name;
+			if (strncmp(name, well_known_symbol, sizeof(well_known_symbol)-1) == 0) {
+				l = strlen(name);
+				pl = l - sizeof(well_known_symbol) - 7;
+				if (pl < 0 || pl > sizeof(derived_prefix)-1)
+					continue;	/* Prefix is wrong length */
+				/* Must end with 8 hex digits */
+				(void) strtoul(name+l-8, &p, 16);
+				if (*p == 0) {
+					strncpy(derived_prefix, name+sizeof(well_known_symbol)-1, pl);
+					*(derived_prefix+pl) = '\0';
+					ncv_prefix = derived_prefix;
+					break;
+				}
+			}
+		}
+	}
+	if (!ncv_prefix) {
+		p = strchr(uts_info.version, ' ');
 		if (p && *(++p) && !strncmp(p, "SMP ", 4))
-			ncv_prefix = SMP_PREFIX;
+			ncv_prefix = "smp_";
+		else
+			ncv_prefix = "";
 	}
 	ncv_plen = strlen(ncv_prefix);
+	if (flag_verbose)
+		printf("Symbol version prefix '%s'\n", ncv_prefix);
 }
 
 static int ncv_strcmp(const char *a, const char *b)
@@ -1238,11 +1269,6 @@ int INSMOD_MAIN(int argc, char **argv)
 		memcpy(m_name, p, len);
 		m_name[len] = '\0';
 	}
-	/*
-	 * Set the SMP prefix if this is an SMP kernel
-	 * and it's not already set.
-	 */
-	set_ncv_prefix(NULL);
 
 	/* Locate the file to be loaded.  */
 	if (!strchr(filename, '/') && !strchr(filename, '.')) {
@@ -1269,6 +1295,12 @@ int INSMOD_MAIN(int argc, char **argv)
 	if (!get_kernel_info(K_SYMBOLS))
 		goto out;
 
+	/*
+	 * Set the genksyms prefix if this is a versioned kernel
+	 * and it's not already set.
+	 */
+	set_ncv_prefix(NULL);
+
 	for (i = 0; i < n_module_stat; ++i) {
 		if (strcmp(module_stat[i].name, m_name) == 0) {
 			error("a module named %s already exists", m_name);
@@ -1419,8 +1451,7 @@ int INSMOD_MAIN(int argc, char **argv)
 #if defined(COMMON_3264) && defined(ONLY_64)
 int insmod_main(int argc, char **argv)
 {
-	struct utsname u;
-	if (!uname (&u) && !strcmp (u.machine, "64"))
+	if (arch64())
 		return insmod_main_64(argc, argv);
 	else
 		return insmod_main_32(argc, argv);
