@@ -27,9 +27,11 @@
  * unified module path handling: Bjorn Ekwall <bj0rn@blox.se> February 1999
  * Rationalize common code for 32/64 bit architectures.
  *   Keith Owens <kaos@ocs.com.au> December 1999.
+ * Add arch64().
+ *   Keith Owens <kaos@ocs.com.au> December 1999.
  */
 
-#ident "$Id: modinfo.c 1.5 Sun, 19 Dec 1999 20:23:21 +1100 keith $"
+#ident "$Id: modinfo.c 1.6 Sun, 26 Dec 1999 20:47:58 +1100 keith $"
 
 #include <stdio.h>
 #include <stdlib.h>
@@ -39,7 +41,6 @@
 #include <unistd.h>
 #include <sys/param.h>
 #include <sys/stat.h>
-#include <sys/utsname.h>
 
 #include "module.h"
 #include "obj.h"
@@ -320,11 +321,10 @@ int SHOW_MODULE_INFO(char *filename, char *fmtstr, int do_parameters)
 #if defined(COMMON_3264) && defined(ONLY_64)
 int show_module_info(char *filename, char *fmtstr, int do_parameters)
 {
-	struct utsname u;
-	if (!uname (&u) && !strcmp (u.machine, "64"))
-		return show_module_info_32(filename, fmtstr, do_parameters);
-	else
+	if (arch64())
 		return show_module_info_64(filename, fmtstr, do_parameters);
+	else
+		return show_module_info_32(filename, fmtstr, do_parameters);
 }
 #endif  /* defined(COMMON_3264) && defined(ONLY_64) */
 
