@@ -169,11 +169,14 @@ static GLOB_LIST *any_options(const char *mod)
 
 /*
  *	Check if a module name is indeed an alias for another module.
- *	Return the alias, or NULL if not found
+ *	Return the alias, or NULL if not found.  devfs requires that
+ *	alias /dev/xyz* /dev/xyz be valid but that can cause a loop.
+ *	Detect this special case and return NULL instead of the same name.
  */
 static const char *any_alias(const char *mod)
 {
 	int i;
+	const char *newmod;
 
 	/*
 	 * Specification: /etc/modules.conf / alias / format
@@ -206,8 +209,13 @@ static const char *any_alias(const char *mod)
 		}
 
 		if (*p == '*' || (*p == '\0' && *q == '\0')) {
-			if (aliases[i].opts && aliases[i].opts->pathc)
-				return aliases[i].opts->pathv[0];
+			if (aliases[i].opts && aliases[i].opts->pathc) {
+				newmod = aliases[i].opts->pathv[0];
+				if (strcmp(mod, newmod))
+					return(newmod);
+				else
+					return(NULL);
+			}
 			break;
 		}
 	}
@@ -278,12 +286,16 @@ static LINK *linkit(int last, void *item, LINK *linkto)
 	return me;
 }
 
+/* Forward definition for probe[all] recusrsion.  */
+static int modprobe_fromlist(int argc, char *argv[], char *type, int loadall);
+
 static DESC *build_desc(const char *name, const char *objname, int allow_alias)
 {
 	DESC *desc;
 	GLOB_LIST *g;
 	NODE *nod = NULL;
 	const char *p;
+	int aliases = 0, ret = 0;
 
 	if (objname && (nod = lookup(in_depfile, objname)) == NULL)
 		return NULL;
@@ -314,6 +326,11 @@ static DESC *build_desc(const char *name, const char *objname, int allow_alias)
 				}
 			}
 		}
+		if (++aliases > 1000) {		/* Arbitrary limit on alias loop */
+			allow_alias = 0;
+			fprintf(stderr, "modprobe: alias loop detected on %s\n", name);
+			break;
+		}
 	}
 	if (!desc->kname)
 		desc->kname = desc->objkey;
@@ -321,6 +338,24 @@ static DESC *build_desc(const char *name, const char *objname, int allow_alias)
 	if (!desc->nod && desc->objkey)
 		desc->nod = lookup_key(in_depfile, desc->objkey);
 
+	/* Iff the result of the above code is not a valid module and is not "off"
+	 * and it is mapped to a string which is different than the original name
+	 * (aliased) then try probe[all] expansion on the result.  devfs does
+	 *   alias           /dev/sg*        /dev/sg
+	 *   probeall        /dev/sg         scsi-hosts sg
+	 * expecting /dev/sg1 alias -> /dev/sg then probeall on /dev/sg.
+	 */
+	if (!desc->nod &&			/* no object */
+	     desc->objkey &&			/* not "off" */
+	     strcmp(name, desc->objkey)) {	/* different name, aliased */
+		if ((g = any_probe_list(desc->objkey)))
+			ret = modprobe_fromlist(g->pathc, g->pathv, NULL, 0);
+		else if ((g = any_probeall_list(desc->objkey)))
+			ret = modprobe_fromlist(g->pathc, g->pathv, NULL, 1);
+		if (ret)
+			desc->objkey = NULL;	/* treat as "off", modprobe_fromlist did the load */
+	}
+
 	return desc;
 }
 
