@@ -24,7 +24,7 @@
  * Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
  */
 
-#ident "$Id: rmmod.c 1.5 Sun, 21 Nov 1999 15:19:53 +1100 keith $"
+#ident "$Id: rmmod.c 1.6 Sat, 25 Dec 1999 14:34:16 +1100 keith $"
 
 #include <errno.h>
 #include <malloc.h>
@@ -106,7 +106,10 @@ int main(int argc, char **argv)
 		for (i = optind; i < argc; ++i) {
 			if (delete_module(argv[i]) < 0) {
 				++ret;
-				perror(argv[1]);
+				if (errno == ENOENT)
+					error("module %s is not loaded", argv[i]);
+				else
+					perror(argv[i]);
 			}
 		}
 		snap_shot(module_names, n_module_names);
