@@ -21,7 +21,7 @@
    along with this program; if not, write to the Free Software Foundation,
    Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.  */
 
-#ident "$Id: obj_arm.c 1.1 Wed, 25 Aug 1999 16:26:49 +1000 keith $"
+#ident "$Id: obj_arm.c 1.2 Mon, 18 Oct 1999 11:04:32 +1000 keith $"
 
 #include <string.h>
 #include <assert.h>
@@ -85,6 +85,7 @@ arch_new_symbol (void)
   struct arm_symbol *sym;
   sym = xmalloc(sizeof(*sym));
   memset(&sym->gotent, 0, sizeof(sym->gotent));
+  memset(&sym->pltent, 0, sizeof(sym->pltent));
   return &sym->root;
 }
 
@@ -151,7 +152,6 @@ arch_apply_relocation (struct obj_file *f,
       /* find the plt entry and initialize it if necessary */
       assert(asym != NULL);
       pe = (struct arm_plt_entry*) &asym->pltent;
-      assert(pe != NULL);
       if (! pe->inited)
 	{
 	  ip = (unsigned long *) (afile->plt->contents + pe->offset);
