@@ -100,6 +100,53 @@ char *config_file = NULL;	/* Which file was actually used */
 time_t config_mtime;
 int root_check_off = 0;		/* Default, modules must be owned by root */
 static char *config_version;	/* Hack for config_add */
+int quick = 0;			/* Option -A */
+
+/*
+ *	Check to see if the existing modules.dep file needs updating,
+ *	based on the timestamps of the modules and the config file.
+ */
+static time_t dep_mtime;
+static int check_update (const char *file, const struct stat *sb, int flag)
+{
+	const char *p;
+	p = strrchr (file, '.');
+	if (p && !strcmp (p, ".o") && sb->st_mtime > dep_mtime)
+		return 1;
+	return 0;
+}
+
+static int need_update (const char *force_ver, const char *base_dir)
+{
+	struct stat statdep, tmp;
+	char dep[PATH_MAX];
+	uname (&uts_info);
+	if (!force_ver)
+		force_ver = uts_info.release;
+	if (!base_dir)
+		base_dir = "";
+
+	if (strlen (force_ver) > 50)
+		/* That's just silly. */
+		return 1;
+
+	if (stat (depfile, &statdep))
+		/* No dependency file yet, so we need to build it. */
+		return 1;
+
+	if (stat ("/etc/modules.conf", &tmp) &&
+	    stat ("/etc/conf.modules", &tmp))
+		return 1;
+
+	if (tmp.st_mtime > statdep.st_mtime)
+		/* Config file is newer. */
+		return 1;
+
+	dep_mtime = statdep.st_mtime;
+	sprintf (dep, "%s/lib/modules/%s", base_dir, force_ver);
+	return ftw (dep, check_update, 2);
+}
+
 
 /*
  *	Strip white char at the end of a string.
@@ -1171,6 +1218,7 @@ static int do_read(int all, char *force_ver, char *base_dir, char *conf_file)
 
 int config_read(int all, char *force_ver, char *base_dir, char *conf_file)
 {
+	int r;
 	if (modpath != NULL)
 		return 0; /* already initialized */
 
@@ -1178,7 +1226,13 @@ int config_read(int all, char *force_ver, char *base_dir, char *conf_file)
 		error("Failed to find kernel name information");
 		return -1;
 	}
-	return do_read(all, force_ver, base_dir, conf_file);
+
+	r = do_read(all, force_ver, base_dir, conf_file);
+
+	if (quick && !r && !need_update (force_ver, base_dir))
+		exit (0);
+
+	return r;
 }
 
 /****************************************************************************/
